#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = u[22] - x[0];

}
void g_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = y[22] + 0.84818460227515158*y[9] + 4.6427621275840973*y[8] + 0.27734492538284594*y[11] - 1.3815573406508483*y[10] + 0.11898572932492885*y[13] - 1.3512526080667266*y[12] + 0.27734492538284605*y[15] - 1.3815573406508488*y[14] - 0.84818460227515158*y[1] - 4.6427621275840973*y[0] - 0.27734492538284594*y[3] + 1.3815573406508483*y[2] - 0.11898572932492885*y[5] + 1.3512526080667266*y[4] - 0.27734492538284605*y[7] + 1.3815573406508488*y[6];
out[1] = y[23] + 4.6427621275840973*y[9] - 0.84818460227515158*y[8] - 1.3815573406508483*y[11] - 0.27734492538284594*y[10] - 1.3512526080667266*y[13] - 0.11898572932492885*y[12] - 1.3815573406508488*y[15] - 0.27734492538284605*y[14] - 4.6427621275840973*y[1] + 0.84818460227515158*y[0] + 1.3815573406508483*y[3] + 0.27734492538284594*y[2] + 1.3512526080667266*y[5] + 0.11898572932492885*y[4] + 1.3815573406508488*y[7] + 0.27734492538284605*y[6];
out[2] = y[24] + 0.27734492538284578*y[9] - 1.3815573406508483*y[8] + 0.84818460227515136*y[11] + 4.6427621275840965*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.11898572932492905*y[15] - 1.3512526080667262*y[14] - 0.27734492538284578*y[1] + 1.3815573406508483*y[0] - 0.84818460227515136*y[3] - 4.6427621275840965*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.11898572932492905*y[7] + 1.3512526080667262*y[6];
out[3] = y[25] - 1.3815573406508483*y[9] - 0.27734492538284578*y[8] + 4.6427621275840965*y[11] - 0.84818460227515136*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] - 1.3512526080667262*y[15] - 0.11898572932492905*y[14] + 1.3815573406508483*y[1] + 0.27734492538284578*y[0] - 4.6427621275840965*y[3] + 0.84818460227515136*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] + 1.3512526080667262*y[7] + 0.11898572932492905*y[6];
out[4] = y[26] + 0.1189857293249293*y[9] - 1.351252608066726*y[8] + 0.27734492538284544*y[11] - 1.3815573406508481*y[10] + 0.84818460227515136*y[13] + 4.6427621275840973*y[12] + 0.27734492538284594*y[15] - 1.3815573406508483*y[14] - 0.1189857293249293*y[1] + 1.351252608066726*y[0] - 0.27734492538284544*y[3] + 1.3815573406508481*y[2] - 0.84818460227515136*y[5] - 4.6427621275840973*y[4] - 0.27734492538284594*y[7] + 1.3815573406508483*y[6];
out[5] = y[27] - 1.351252608066726*y[9] - 0.1189857293249293*y[8] - 1.3815573406508481*y[11] - 0.27734492538284544*y[10] + 4.6427621275840973*y[13] - 0.84818460227515136*y[12] - 1.3815573406508483*y[15] - 0.27734492538284594*y[14] + 1.351252608066726*y[1] + 0.1189857293249293*y[0] + 1.3815573406508481*y[3] + 0.27734492538284544*y[2] - 4.6427621275840973*y[5] + 0.84818460227515136*y[4] + 1.3815573406508483*y[7] + 0.27734492538284594*y[6];
out[6] = 0.27734492538284572*y[9] - 1.3815573406508488*y[8] + 0.11898572932492921*y[11] - 1.3512526080667266*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.84818460227515124*y[15] + 4.6427621275840973*y[14] - 0.27734492538284572*y[1] + 1.3815573406508488*y[0] - 0.11898572932492921*y[3] + 1.3512526080667266*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.84818460227515124*y[7] - 4.6677621275840977*y[6];
out[7] = -1.3815573406508488*y[9] - 0.27734492538284572*y[8] - 1.3512526080667266*y[11] - 0.11898572932492921*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] + 4.6427621275840973*y[15] - 0.84818460227515124*y[14] + 1.3815573406508488*y[1] + 0.27734492538284572*y[0] + 1.3512526080667266*y[3] + 0.11898572932492921*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] - 4.6677621275840977*y[7] + 0.84818460227515124*y[6];
out[8] = 0.25471335405424661*u[1] + 0.063678338513561653*u[0] - 0.25471335405424661*u[5] - 0.063678338513561653*u[4] - 22.907008131686908*y[9] - 10.157468009937038*y[8] - 0.27734492538284594*y[11] + 1.3815573406508483*y[10] - 0.11898572932492885*y[13] + 1.3512526080667266*y[12] + 21.781478604028912*y[15] + 6.8962632230037881*y[14] + 0.84818460227515158*y[1] + 4.6427621275840973*y[0] + 0.27734492538284594*y[3] - 1.3815573406508483*y[2] + 0.11898572932492885*y[5] - 1.3512526080667266*y[4] + 0.27734492538284605*y[7] - 1.3815573406508488*y[6];
out[9] = 0.063678338513561653*u[1] - 0.25471335405424661*u[0] - 0.063678338513561653*u[5] + 0.25471335405424661*u[4] - 10.157468009937038*y[9] + 22.907008131686908*y[8] + 1.3815573406508483*y[11] + 0.27734492538284594*y[10] + 1.3512526080667266*y[13] + 0.11898572932492885*y[12] + 6.8962632230037881*y[15] - 21.781478604028912*y[14] + 4.6427621275840973*y[1] - 0.84818460227515158*y[0] - 1.3815573406508483*y[3] - 0.27734492538284594*y[2] - 1.3512526080667266*y[5] - 0.11898572932492885*y[4] - 1.3815573406508488*y[7] - 0.27734492538284605*y[6];
out[10] = -0.25471335405424661*u[1] - 0.063678338513561653*u[0] + 0.25471335405424661*u[3] + 0.063678338513561653*u[2] - 0.27734492538284578*y[9] + 1.3815573406508483*y[8] - 22.907008131686908*y[11] - 10.157468009937036*y[10] - 0.27734492538284605*y[13] + 1.3815573406508481*y[12] + 21.939837800086828*y[15] + 6.8659584904196658*y[14] + 0.27734492538284578*y[1] - 1.3815573406508483*y[0] + 0.84818460227515136*y[3] + 4.6427621275840965*y[2] + 0.27734492538284605*y[5] - 1.3815573406508481*y[4] + 0.11898572932492905*y[7] - 1.3512526080667262*y[6];
out[11] = -0.063678338513561653*u[1] + 0.25471335405424661*u[0] + 0.063678338513561653*u[3] - 0.25471335405424661*u[2] + 1.3815573406508483*y[9] + 0.27734492538284578*y[8] - 10.157468009937036*y[11] + 22.907008131686908*y[10] + 1.3815573406508481*y[13] + 0.27734492538284605*y[12] + 6.8659584904196658*y[15] - 21.939837800086828*y[14] - 1.3815573406508483*y[1] - 0.27734492538284578*y[0] + 4.6427621275840965*y[3] - 0.84818460227515136*y[2] - 1.3815573406508481*y[5] - 0.27734492538284605*y[4] - 1.3512526080667262*y[7] - 0.11898572932492905*y[6];
out[12] = -0.25471335405424661*u[3] - 0.063678338513561653*u[2] + 0.25471335405424661*u[5] + 0.063678338513561653*u[4] - 0.1189857293249293*y[9] + 1.351252608066726*y[8] - 0.27734492538284544*y[11] + 1.3815573406508481*y[10] - 22.907008131686908*y[13] - 10.157468009937038*y[12] + 21.781478604028912*y[15] + 6.8962632230037872*y[14] + 0.1189857293249293*y[1] - 1.351252608066726*y[0] + 0.27734492538284544*y[3] - 1.3815573406508481*y[2] + 0.84818460227515136*y[5] + 4.6427621275840973*y[4] + 0.27734492538284594*y[7] - 1.3815573406508483*y[6];
out[13] = -0.063678338513561653*u[3] + 0.25471335405424661*u[2] + 0.063678338513561653*u[5] - 0.25471335405424661*u[4] + 1.351252608066726*y[9] + 0.1189857293249293*y[8] + 1.3815573406508481*y[11] + 0.27734492538284544*y[10] - 10.157468009937038*y[13] + 22.907008131686908*y[12] + 6.8962632230037872*y[15] - 21.781478604028912*y[14] - 1.351252608066726*y[1] - 0.1189857293249293*y[0] - 1.3815573406508481*y[3] - 0.27734492538284544*y[2] + 4.6427621275840973*y[5] - 0.84818460227515136*y[4] - 1.3815573406508483*y[7] - 0.27734492538284594*y[6];
out[14] = 21.781478604028912*y[9] + 6.8962632230037881*y[8] + 21.939837800086828*y[11] + 6.8659584904196658*y[10] + 21.781478604028912*y[13] + 6.8962632230037872*y[12] - 67.02465519051043*y[15] - 21.52021310797625*y[14] + 0.27734492538284572*y[1] - 1.3815573406508488*y[0] + 0.11898572932492921*y[3] - 1.3512526080667266*y[2] + 0.27734492538284605*y[5] - 1.3815573406508481*y[4] + 0.84818460227515124*y[7] + 4.6427621275840973*y[6];
out[15] = 6.8962632230037881*y[9] - 21.781478604028912*y[8] + 6.8659584904196658*y[11] - 21.939837800086828*y[10] + 6.8962632230037872*y[13] - 21.781478604028912*y[12] - 21.52021310797625*y[15] + 67.02465519051043*y[14] - 1.3815573406508488*y[1] - 0.27734492538284572*y[0] - 1.3512526080667266*y[3] - 0.11898572932492921*y[2] - 1.3815573406508481*y[5] - 0.27734492538284605*y[4] + 4.6427621275840973*y[7] - 0.84818460227515124*y[6];
out[16] = -y[16] + 0.0058823529411764714*u[1] + 0.0014705882352941178*u[0] - 0.0029411764705882357*u[3] - 0.00073529411764705892*u[2] - 0.0029411764705882357*u[5] - 0.00073529411764705892*u[4] - 0.25471335405424661*y[9] - 0.063678338513561653*y[8] + 0.25471335405424661*y[11] + 0.063678338513561653*y[10];
out[17] = -y[17] + 0.0014705882352941178*u[1] - 0.0058823529411764714*u[0] - 0.00073529411764705892*u[3] + 0.0029411764705882357*u[2] - 0.00073529411764705892*u[5] + 0.0029411764705882357*u[4] - 0.063678338513561653*y[9] + 0.25471335405424661*y[8] + 0.063678338513561653*y[11] - 0.25471335405424661*y[10];
out[18] = -y[18] - 0.0029411764705882357*u[1] - 0.00073529411764705892*u[0] + 0.0058823529411764714*u[3] + 0.0014705882352941178*u[2] - 0.0029411764705882357*u[5] - 0.00073529411764705892*u[4] - 0.25471335405424661*y[11] - 0.063678338513561653*y[10] + 0.25471335405424661*y[13] + 0.063678338513561653*y[12];
out[19] = -y[19] - 0.00073529411764705892*u[1] + 0.0029411764705882357*u[0] + 0.0014705882352941178*u[3] - 0.0058823529411764714*u[2] - 0.00073529411764705892*u[5] + 0.0029411764705882357*u[4] - 0.063678338513561653*y[11] + 0.25471335405424661*y[10] + 0.063678338513561653*y[13] - 0.25471335405424661*y[12];
out[20] = -y[20] - 0.0029411764705882357*u[1] - 0.00073529411764705892*u[0] - 0.0029411764705882357*u[3] - 0.00073529411764705892*u[2] + 0.0058823529411764714*u[5] + 0.0014705882352941178*u[4] + 0.25471335405424661*y[9] + 0.063678338513561653*y[8] - 0.25471335405424661*y[13] - 0.063678338513561653*y[12];
out[21] = -y[21] - 0.00073529411764705892*u[1] + 0.0029411764705882357*u[0] - 0.00073529411764705892*u[3] + 0.0029411764705882357*u[2] + 0.0014705882352941178*u[5] - 0.0058823529411764714*u[4] + 0.063678338513561653*y[9] - 0.25471335405424661*y[8] - 0.063678338513561653*y[13] + 0.25471335405424661*y[12];
out[22] = y[23]*y[1] - y[23]*y[7] + y[22]*y[0] - y[22]*y[6] - u[16];
out[23] = y[25]*y[3] - y[25]*y[7] + y[24]*y[2] - y[24]*y[6] - u[18];
out[24] = y[27]*y[5] - y[27]*y[7] + y[26]*y[4] - y[26]*y[6] - u[20];
out[25] = -y[23]*y[0] + y[23]*y[6] + y[22]*y[1] - y[22]*y[7] - u[17];
out[26] = -y[25]*y[2] + y[25]*y[6] + y[24]*y[3] - y[24]*y[7] - u[19];
out[27] = -y[27]*y[4] + y[27]*y[6] + y[26]*y[5] - y[26]*y[7] - u[21];
out[28] = y[22] + y[24] + y[26] + y[28];
out[29] = y[23] + y[25] + y[27] + y[29];

}
void f_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = u[22] - x[0];

}
void h_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.84818460227515158*y[9] + 4.6427621275840973*y[8] + 0.27734492538284594*y[11] - 1.3815573406508483*y[10] + 0.11898572932492885*y[13] - 1.3512526080667266*y[12] + 0.27734492538284605*y[15] - 1.3815573406508488*y[14] - 0.84818460227515158*y[1] - 4.6427621275840973*y[0] - 0.27734492538284594*y[3] + 1.3815573406508483*y[2] - 0.11898572932492885*y[5] + 1.3512526080667266*y[4] - 0.27734492538284605*y[7] + 1.3815573406508488*y[6];
out[1] = 4.6427621275840973*y[9] - 0.84818460227515158*y[8] - 1.3815573406508483*y[11] - 0.27734492538284594*y[10] - 1.3512526080667266*y[13] - 0.11898572932492885*y[12] - 1.3815573406508488*y[15] - 0.27734492538284605*y[14] - 4.6427621275840973*y[1] + 0.84818460227515158*y[0] + 1.3815573406508483*y[3] + 0.27734492538284594*y[2] + 1.3512526080667266*y[5] + 0.11898572932492885*y[4] + 1.3815573406508488*y[7] + 0.27734492538284605*y[6];
out[2] = 0.27734492538284578*y[9] - 1.3815573406508483*y[8] + 0.84818460227515136*y[11] + 4.6427621275840965*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.11898572932492905*y[15] - 1.3512526080667262*y[14] - 0.27734492538284578*y[1] + 1.3815573406508483*y[0] - 0.84818460227515136*y[3] - 4.6427621275840965*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.11898572932492905*y[7] + 1.3512526080667262*y[6];
out[3] = -1.3815573406508483*y[9] - 0.27734492538284578*y[8] + 4.6427621275840965*y[11] - 0.84818460227515136*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] - 1.3512526080667262*y[15] - 0.11898572932492905*y[14] + 1.3815573406508483*y[1] + 0.27734492538284578*y[0] - 4.6427621275840965*y[3] + 0.84818460227515136*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] + 1.3512526080667262*y[7] + 0.11898572932492905*y[6];
out[4] = 0.1189857293249293*y[9] - 1.351252608066726*y[8] + 0.27734492538284544*y[11] - 1.3815573406508481*y[10] + 0.84818460227515136*y[13] + 4.6427621275840973*y[12] + 0.27734492538284594*y[15] - 1.3815573406508483*y[14] - 0.1189857293249293*y[1] + 1.351252608066726*y[0] - 0.27734492538284544*y[3] + 1.3815573406508481*y[2] - 0.84818460227515136*y[5] - 4.6427621275840973*y[4] - 0.27734492538284594*y[7] + 1.3815573406508483*y[6];
out[5] = -1.351252608066726*y[9] - 0.1189857293249293*y[8] - 1.3815573406508481*y[11] - 0.27734492538284544*y[10] + 4.6427621275840973*y[13] - 0.84818460227515136*y[12] - 1.3815573406508483*y[15] - 0.27734492538284594*y[14] + 1.351252608066726*y[1] + 0.1189857293249293*y[0] + 1.3815573406508481*y[3] + 0.27734492538284544*y[2] - 4.6427621275840973*y[5] + 0.84818460227515136*y[4] + 1.3815573406508483*y[7] + 0.27734492538284594*y[6];
out[6] = 0.27734492538284572*y[9] - 1.3815573406508488*y[8] + 0.11898572932492921*y[11] - 1.3512526080667266*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.84818460227515124*y[15] + 4.6427621275840973*y[14] - 0.27734492538284572*y[1] + 1.3815573406508488*y[0] - 0.11898572932492921*y[3] + 1.3512526080667266*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.84818460227515124*y[7] - 4.6427621275840973*y[6];
out[7] = -1.3815573406508488*y[9] - 0.27734492538284572*y[8] - 1.3512526080667266*y[11] - 0.11898572932492921*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] + 4.6427621275840973*y[15] - 0.84818460227515124*y[14] + 1.3815573406508488*y[1] + 0.27734492538284572*y[0] + 1.3512526080667266*y[3] + 0.11898572932492921*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] - 4.6427621275840973*y[7] + 0.84818460227515124*y[6];
out[8] = sqrt(pow(u[1], 2) + pow(u[0], 2));
out[9] = sqrt(pow(u[3], 2) + pow(u[2], 2));
out[10] = sqrt(pow(u[5], 2) + pow(u[4], 2));
out[11] = sqrt(pow(y[1], 2) + pow(y[0], 2));
out[12] = sqrt(pow(y[3], 2) + pow(y[2], 2));
out[13] = sqrt(pow(y[5], 2) + pow(y[4], 2));
out[14] = sqrt(pow(y[7], 2) + pow(y[6], 2));
out[15] = sqrt(pow(y[9], 2) + pow(y[8], 2));
out[16] = sqrt(pow(y[11], 2) + pow(y[10], 2));
out[17] = sqrt(pow(y[13], 2) + pow(y[12], 2));
out[18] = sqrt(pow(y[15], 2) + pow(y[14], 2));

}
void g_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = y[22] + 0.84818460227515158*y[9] + 4.6427621275840973*y[8] + 0.27734492538284594*y[11] - 1.3815573406508483*y[10] + 0.11898572932492885*y[13] - 1.3512526080667266*y[12] + 0.27734492538284605*y[15] - 1.3815573406508488*y[14] - 0.84818460227515158*y[1] - 4.6427621275840973*y[0] - 0.27734492538284594*y[3] + 1.3815573406508483*y[2] - 0.11898572932492885*y[5] + 1.3512526080667266*y[4] - 0.27734492538284605*y[7] + 1.3815573406508488*y[6];
out[1] = y[23] + 4.6427621275840973*y[9] - 0.84818460227515158*y[8] - 1.3815573406508483*y[11] - 0.27734492538284594*y[10] - 1.3512526080667266*y[13] - 0.11898572932492885*y[12] - 1.3815573406508488*y[15] - 0.27734492538284605*y[14] - 4.6427621275840973*y[1] + 0.84818460227515158*y[0] + 1.3815573406508483*y[3] + 0.27734492538284594*y[2] + 1.3512526080667266*y[5] + 0.11898572932492885*y[4] + 1.3815573406508488*y[7] + 0.27734492538284605*y[6];
out[2] = y[24] + 0.27734492538284578*y[9] - 1.3815573406508483*y[8] + 0.84818460227515136*y[11] + 4.6427621275840965*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.11898572932492905*y[15] - 1.3512526080667262*y[14] - 0.27734492538284578*y[1] + 1.3815573406508483*y[0] - 0.84818460227515136*y[3] - 4.6427621275840965*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.11898572932492905*y[7] + 1.3512526080667262*y[6];
out[3] = y[25] - 1.3815573406508483*y[9] - 0.27734492538284578*y[8] + 4.6427621275840965*y[11] - 0.84818460227515136*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] - 1.3512526080667262*y[15] - 0.11898572932492905*y[14] + 1.3815573406508483*y[1] + 0.27734492538284578*y[0] - 4.6427621275840965*y[3] + 0.84818460227515136*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] + 1.3512526080667262*y[7] + 0.11898572932492905*y[6];
out[4] = y[26] + 0.1189857293249293*y[9] - 1.351252608066726*y[8] + 0.27734492538284544*y[11] - 1.3815573406508481*y[10] + 0.84818460227515136*y[13] + 4.6427621275840973*y[12] + 0.27734492538284594*y[15] - 1.3815573406508483*y[14] - 0.1189857293249293*y[1] + 1.351252608066726*y[0] - 0.27734492538284544*y[3] + 1.3815573406508481*y[2] - 0.84818460227515136*y[5] - 4.6427621275840973*y[4] - 0.27734492538284594*y[7] + 1.3815573406508483*y[6];
out[5] = y[27] - 1.351252608066726*y[9] - 0.1189857293249293*y[8] - 1.3815573406508481*y[11] - 0.27734492538284544*y[10] + 4.6427621275840973*y[13] - 0.84818460227515136*y[12] - 1.3815573406508483*y[15] - 0.27734492538284594*y[14] + 1.351252608066726*y[1] + 0.1189857293249293*y[0] + 1.3815573406508481*y[3] + 0.27734492538284544*y[2] - 4.6427621275840973*y[5] + 0.84818460227515136*y[4] + 1.3815573406508483*y[7] + 0.27734492538284594*y[6];
out[6] = 0.27734492538284572*y[9] - 1.3815573406508488*y[8] + 0.11898572932492921*y[11] - 1.3512526080667266*y[10] + 0.27734492538284605*y[13] - 1.3815573406508481*y[12] + 0.84818460227515124*y[15] + 4.6427621275840973*y[14] - 0.27734492538284572*y[1] + 1.3815573406508488*y[0] - 0.11898572932492921*y[3] + 1.3512526080667266*y[2] - 0.27734492538284605*y[5] + 1.3815573406508481*y[4] - 0.84818460227515124*y[7] - 4.6677621275840977*y[6];
out[7] = -1.3815573406508488*y[9] - 0.27734492538284572*y[8] - 1.3512526080667266*y[11] - 0.11898572932492921*y[10] - 1.3815573406508481*y[13] - 0.27734492538284605*y[12] + 4.6427621275840973*y[15] - 0.84818460227515124*y[14] + 1.3815573406508488*y[1] + 0.27734492538284572*y[0] + 1.3512526080667266*y[3] + 0.11898572932492921*y[2] + 1.3815573406508481*y[5] + 0.27734492538284605*y[4] - 4.6677621275840977*y[7] + 0.84818460227515124*y[6];
out[8] = 0.25471335405424661*u[1] + 0.063678338513561653*u[0] - 0.25471335405424661*u[5] - 0.063678338513561653*u[4] - 22.907008131686908*y[9] - 10.157468009937038*y[8] - 0.27734492538284594*y[11] + 1.3815573406508483*y[10] - 0.11898572932492885*y[13] + 1.3512526080667266*y[12] + 21.781478604028912*y[15] + 6.8962632230037881*y[14] + 0.84818460227515158*y[1] + 4.6427621275840973*y[0] + 0.27734492538284594*y[3] - 1.3815573406508483*y[2] + 0.11898572932492885*y[5] - 1.3512526080667266*y[4] + 0.27734492538284605*y[7] - 1.3815573406508488*y[6];
out[9] = 0.063678338513561653*u[1] - 0.25471335405424661*u[0] - 0.063678338513561653*u[5] + 0.25471335405424661*u[4] - 10.157468009937038*y[9] + 22.907008131686908*y[8] + 1.3815573406508483*y[11] + 0.27734492538284594*y[10] + 1.3512526080667266*y[13] + 0.11898572932492885*y[12] + 6.8962632230037881*y[15] - 21.781478604028912*y[14] + 4.6427621275840973*y[1] - 0.84818460227515158*y[0] - 1.3815573406508483*y[3] - 0.27734492538284594*y[2] - 1.3512526080667266*y[5] - 0.11898572932492885*y[4] - 1.3815573406508488*y[7] - 0.27734492538284605*y[6];
out[10] = -0.25471335405424661*u[1] - 0.063678338513561653*u[0] + 0.25471335405424661*u[3] + 0.063678338513561653*u[2] - 0.27734492538284578*y[9] + 1.3815573406508483*y[8] - 22.907008131686908*y[11] - 10.157468009937036*y[10] - 0.27734492538284605*y[13] + 1.3815573406508481*y[12] + 21.939837800086828*y[15] + 6.8659584904196658*y[14] + 0.27734492538284578*y[1] - 1.3815573406508483*y[0] + 0.84818460227515136*y[3] + 4.6427621275840965*y[2] + 0.27734492538284605*y[5] - 1.3815573406508481*y[4] + 0.11898572932492905*y[7] - 1.3512526080667262*y[6];
out[11] = -0.063678338513561653*u[1] + 0.25471335405424661*u[0] + 0.063678338513561653*u[3] - 0.25471335405424661*u[2] + 1.3815573406508483*y[9] + 0.27734492538284578*y[8] - 10.157468009937036*y[11] + 22.907008131686908*y[10] + 1.3815573406508481*y[13] + 0.27734492538284605*y[12] + 6.8659584904196658*y[15] - 21.939837800086828*y[14] - 1.3815573406508483*y[1] - 0.27734492538284578*y[0] + 4.6427621275840965*y[3] - 0.84818460227515136*y[2] - 1.3815573406508481*y[5] - 0.27734492538284605*y[4] - 1.3512526080667262*y[7] - 0.11898572932492905*y[6];
out[12] = -0.25471335405424661*u[3] - 0.063678338513561653*u[2] + 0.25471335405424661*u[5] + 0.063678338513561653*u[4] - 0.1189857293249293*y[9] + 1.351252608066726*y[8] - 0.27734492538284544*y[11] + 1.3815573406508481*y[10] - 22.907008131686908*y[13] - 10.157468009937038*y[12] + 21.781478604028912*y[15] + 6.8962632230037872*y[14] + 0.1189857293249293*y[1] - 1.351252608066726*y[0] + 0.27734492538284544*y[3] - 1.3815573406508481*y[2] + 0.84818460227515136*y[5] + 4.6427621275840973*y[4] + 0.27734492538284594*y[7] - 1.3815573406508483*y[6];
out[13] = -0.063678338513561653*u[3] + 0.25471335405424661*u[2] + 0.063678338513561653*u[5] - 0.25471335405424661*u[4] + 1.351252608066726*y[9] + 0.1189857293249293*y[8] + 1.3815573406508481*y[11] + 0.27734492538284544*y[10] - 10.157468009937038*y[13] + 22.907008131686908*y[12] + 6.8962632230037872*y[15] - 21.781478604028912*y[14] - 1.351252608066726*y[1] - 0.1189857293249293*y[0] - 1.3815573406508481*y[3] - 0.27734492538284544*y[2] + 4.6427621275840973*y[5] - 0.84818460227515136*y[4] - 1.3815573406508483*y[7] - 0.27734492538284594*y[6];
out[14] = 21.781478604028912*y[9] + 6.8962632230037881*y[8] + 21.939837800086828*y[11] + 6.8659584904196658*y[10] + 21.781478604028912*y[13] + 6.8962632230037872*y[12] - 67.02465519051043*y[15] - 21.52021310797625*y[14] + 0.27734492538284572*y[1] - 1.3815573406508488*y[0] + 0.11898572932492921*y[3] - 1.3512526080667266*y[2] + 0.27734492538284605*y[5] - 1.3815573406508481*y[4] + 0.84818460227515124*y[7] + 4.6427621275840973*y[6];
out[15] = 6.8962632230037881*y[9] - 21.781478604028912*y[8] + 6.8659584904196658*y[11] - 21.939837800086828*y[10] + 6.8962632230037872*y[13] - 21.781478604028912*y[12] - 21.52021310797625*y[15] + 67.02465519051043*y[14] - 1.3815573406508488*y[1] - 0.27734492538284572*y[0] - 1.3512526080667266*y[3] - 0.11898572932492921*y[2] - 1.3815573406508481*y[5] - 0.27734492538284605*y[4] + 4.6427621275840973*y[7] - 0.84818460227515124*y[6];
out[16] = -y[16] + 0.0058823529411764714*u[1] + 0.0014705882352941178*u[0] - 0.0029411764705882357*u[3] - 0.00073529411764705892*u[2] - 0.0029411764705882357*u[5] - 0.00073529411764705892*u[4] - 0.25471335405424661*y[9] - 0.063678338513561653*y[8] + 0.25471335405424661*y[11] + 0.063678338513561653*y[10];
out[17] = -y[17] + 0.0014705882352941178*u[1] - 0.0058823529411764714*u[0] - 0.00073529411764705892*u[3] + 0.0029411764705882357*u[2] - 0.00073529411764705892*u[5] + 0.0029411764705882357*u[4] - 0.063678338513561653*y[9] + 0.25471335405424661*y[8] + 0.063678338513561653*y[11] - 0.25471335405424661*y[10];
out[18] = -y[18] - 0.0029411764705882357*u[1] - 0.00073529411764705892*u[0] + 0.0058823529411764714*u[3] + 0.0014705882352941178*u[2] - 0.0029411764705882357*u[5] - 0.00073529411764705892*u[4] - 0.25471335405424661*y[11] - 0.063678338513561653*y[10] + 0.25471335405424661*y[13] + 0.063678338513561653*y[12];
out[19] = -y[19] - 0.00073529411764705892*u[1] + 0.0029411764705882357*u[0] + 0.0014705882352941178*u[3] - 0.0058823529411764714*u[2] - 0.00073529411764705892*u[5] + 0.0029411764705882357*u[4] - 0.063678338513561653*y[11] + 0.25471335405424661*y[10] + 0.063678338513561653*y[13] - 0.25471335405424661*y[12];
out[20] = -y[20] - 0.0029411764705882357*u[1] - 0.00073529411764705892*u[0] - 0.0029411764705882357*u[3] - 0.00073529411764705892*u[2] + 0.0058823529411764714*u[5] + 0.0014705882352941178*u[4] + 0.25471335405424661*y[9] + 0.063678338513561653*y[8] - 0.25471335405424661*y[13] - 0.063678338513561653*y[12];
out[21] = -y[21] - 0.00073529411764705892*u[1] + 0.0029411764705882357*u[0] - 0.00073529411764705892*u[3] + 0.0029411764705882357*u[2] + 0.0014705882352941178*u[5] - 0.0058823529411764714*u[4] + 0.063678338513561653*y[9] - 0.25471335405424661*y[8] - 0.063678338513561653*y[13] + 0.25471335405424661*y[12];
out[22] = y[23]*y[1] - y[23]*y[7] + y[22]*y[0] - y[22]*y[6] - u[16];
out[23] = y[25]*y[3] - y[25]*y[7] + y[24]*y[2] - y[24]*y[6] - u[18];
out[24] = y[27]*y[5] - y[27]*y[7] + y[26]*y[4] - y[26]*y[6] - u[20];
out[25] = -y[23]*y[0] + y[23]*y[6] + y[22]*y[1] - y[22]*y[7] - u[17];
out[26] = -y[25]*y[2] + y[25]*y[6] + y[24]*y[3] - y[24]*y[7] - u[19];
out[27] = -y[27]*y[4] + y[27]*y[6] + y[26]*y[5] - y[26]*y[7] - u[21];
out[28] = y[22] + y[24] + y[26] + y[28];
out[29] = y[23] + y[25] + y[27] + y[29];

}
void de_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[714] = y[22];
out[715] = y[23];
out[720] = -y[22];
out[721] = -y[23];
out[736] = y[0] - y[6];
out[737] = y[1] - y[7];
out[747] = y[24];
out[748] = y[25];
out[751] = -y[24];
out[752] = -y[25];
out[769] = y[2] - y[6];
out[770] = y[3] - y[7];
out[780] = y[26];
out[781] = y[27];
out[782] = -y[26];
out[783] = -y[27];
out[802] = y[4] - y[6];
out[803] = y[5] - y[7];
out[807] = -y[23];
out[808] = y[22];
out[813] = y[23];
out[814] = -y[22];
out[829] = y[1] - y[7];
out[830] = -y[0] + y[6];
out[840] = -y[25];
out[841] = y[24];
out[844] = y[25];
out[845] = -y[24];
out[862] = y[3] - y[7];
out[863] = -y[2] + y[6];
out[873] = -y[27];
out[874] = y[26];
out[875] = y[27];
out[876] = -y[26];
out[895] = y[5] - y[7];
out[896] = -y[4] + y[6];

}

void de_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -1;
out[32] = -4.6427621275840973;
out[33] = -0.84818460227515158;
out[34] = 1.3815573406508483;
out[35] = -0.27734492538284594;
out[36] = 1.3512526080667266;
out[37] = -0.11898572932492885;
out[38] = 1.3815573406508488;
out[39] = -0.27734492538284605;
out[40] = 4.6427621275840973;
out[41] = 0.84818460227515158;
out[42] = -1.3815573406508483;
out[43] = 0.27734492538284594;
out[44] = -1.3512526080667266;
out[45] = 0.11898572932492885;
out[46] = -1.3815573406508488;
out[47] = 0.27734492538284605;
out[54] = 1;
out[63] = 0.84818460227515158;
out[64] = -4.6427621275840973;
out[65] = 0.27734492538284594;
out[66] = 1.3815573406508483;
out[67] = 0.11898572932492885;
out[68] = 1.3512526080667266;
out[69] = 0.27734492538284605;
out[70] = 1.3815573406508488;
out[71] = -0.84818460227515158;
out[72] = 4.6427621275840973;
out[73] = -0.27734492538284594;
out[74] = -1.3815573406508483;
out[75] = -0.11898572932492885;
out[76] = -1.3512526080667266;
out[77] = -0.27734492538284605;
out[78] = -1.3815573406508488;
out[86] = 1;
out[94] = 1.3815573406508483;
out[95] = -0.27734492538284578;
out[96] = -4.6427621275840965;
out[97] = -0.84818460227515136;
out[98] = 1.3815573406508481;
out[99] = -0.27734492538284605;
out[100] = 1.3512526080667262;
out[101] = -0.11898572932492905;
out[102] = -1.3815573406508483;
out[103] = 0.27734492538284578;
out[104] = 4.6427621275840965;
out[105] = 0.84818460227515136;
out[106] = -1.3815573406508481;
out[107] = 0.27734492538284605;
out[108] = -1.3512526080667262;
out[109] = 0.11898572932492905;
out[118] = 1;
out[125] = 0.27734492538284578;
out[126] = 1.3815573406508483;
out[127] = 0.84818460227515136;
out[128] = -4.6427621275840965;
out[129] = 0.27734492538284605;
out[130] = 1.3815573406508481;
out[131] = 0.11898572932492905;
out[132] = 1.3512526080667262;
out[133] = -0.27734492538284578;
out[134] = -1.3815573406508483;
out[135] = -0.84818460227515136;
out[136] = 4.6427621275840965;
out[137] = -0.27734492538284605;
out[138] = -1.3815573406508481;
out[139] = -0.11898572932492905;
out[140] = -1.3512526080667262;
out[150] = 1;
out[156] = 1.351252608066726;
out[157] = -0.1189857293249293;
out[158] = 1.3815573406508481;
out[159] = -0.27734492538284544;
out[160] = -4.6427621275840973;
out[161] = -0.84818460227515136;
out[162] = 1.3815573406508483;
out[163] = -0.27734492538284594;
out[164] = -1.351252608066726;
out[165] = 0.1189857293249293;
out[166] = -1.3815573406508481;
out[167] = 0.27734492538284544;
out[168] = 4.6427621275840973;
out[169] = 0.84818460227515136;
out[170] = -1.3815573406508483;
out[171] = 0.27734492538284594;
out[182] = 1;
out[187] = 0.1189857293249293;
out[188] = 1.351252608066726;
out[189] = 0.27734492538284544;
out[190] = 1.3815573406508481;
out[191] = 0.84818460227515136;
out[192] = -4.6427621275840973;
out[193] = 0.27734492538284594;
out[194] = 1.3815573406508483;
out[195] = -0.1189857293249293;
out[196] = -1.351252608066726;
out[197] = -0.27734492538284544;
out[198] = -1.3815573406508481;
out[199] = -0.84818460227515136;
out[200] = 4.6427621275840973;
out[201] = -0.27734492538284594;
out[202] = -1.3815573406508483;
out[214] = 1;
out[218] = 1.3815573406508488;
out[219] = -0.27734492538284572;
out[220] = 1.3512526080667266;
out[221] = -0.11898572932492921;
out[222] = 1.3815573406508481;
out[223] = -0.27734492538284605;
out[224] = -4.6677621275840977;
out[225] = -0.84818460227515124;
out[226] = -1.3815573406508488;
out[227] = 0.27734492538284572;
out[228] = -1.3512526080667266;
out[229] = 0.11898572932492921;
out[230] = -1.3815573406508481;
out[231] = 0.27734492538284605;
out[232] = 4.6427621275840973;
out[233] = 0.84818460227515124;
out[249] = 0.27734492538284572;
out[250] = 1.3815573406508488;
out[251] = 0.11898572932492921;
out[252] = 1.3512526080667266;
out[253] = 0.27734492538284605;
out[254] = 1.3815573406508481;
out[255] = 0.84818460227515124;
out[256] = -4.6677621275840977;
out[257] = -0.27734492538284572;
out[258] = -1.3815573406508488;
out[259] = -0.11898572932492921;
out[260] = -1.3512526080667266;
out[261] = -0.27734492538284605;
out[262] = -1.3815573406508481;
out[263] = -0.84818460227515124;
out[264] = 4.6427621275840973;
out[280] = 4.6427621275840973;
out[281] = 0.84818460227515158;
out[282] = -1.3815573406508483;
out[283] = 0.27734492538284594;
out[284] = -1.3512526080667266;
out[285] = 0.11898572932492885;
out[286] = -1.3815573406508488;
out[287] = 0.27734492538284605;
out[288] = -10.157468009937038;
out[289] = -22.907008131686908;
out[290] = 1.3815573406508483;
out[291] = -0.27734492538284594;
out[292] = 1.3512526080667266;
out[293] = -0.11898572932492885;
out[294] = 6.8962632230037881;
out[295] = 21.781478604028912;
out[311] = -0.84818460227515158;
out[312] = 4.6427621275840973;
out[313] = -0.27734492538284594;
out[314] = -1.3815573406508483;
out[315] = -0.11898572932492885;
out[316] = -1.3512526080667266;
out[317] = -0.27734492538284605;
out[318] = -1.3815573406508488;
out[319] = 22.907008131686908;
out[320] = -10.157468009937038;
out[321] = 0.27734492538284594;
out[322] = 1.3815573406508483;
out[323] = 0.11898572932492885;
out[324] = 1.3512526080667266;
out[325] = -21.781478604028912;
out[326] = 6.8962632230037881;
out[342] = -1.3815573406508483;
out[343] = 0.27734492538284578;
out[344] = 4.6427621275840965;
out[345] = 0.84818460227515136;
out[346] = -1.3815573406508481;
out[347] = 0.27734492538284605;
out[348] = -1.3512526080667262;
out[349] = 0.11898572932492905;
out[350] = 1.3815573406508483;
out[351] = -0.27734492538284578;
out[352] = -10.157468009937036;
out[353] = -22.907008131686908;
out[354] = 1.3815573406508481;
out[355] = -0.27734492538284605;
out[356] = 6.8659584904196658;
out[357] = 21.939837800086828;
out[373] = -0.27734492538284578;
out[374] = -1.3815573406508483;
out[375] = -0.84818460227515136;
out[376] = 4.6427621275840965;
out[377] = -0.27734492538284605;
out[378] = -1.3815573406508481;
out[379] = -0.11898572932492905;
out[380] = -1.3512526080667262;
out[381] = 0.27734492538284578;
out[382] = 1.3815573406508483;
out[383] = 22.907008131686908;
out[384] = -10.157468009937036;
out[385] = 0.27734492538284605;
out[386] = 1.3815573406508481;
out[387] = -21.939837800086828;
out[388] = 6.8659584904196658;
out[404] = -1.351252608066726;
out[405] = 0.1189857293249293;
out[406] = -1.3815573406508481;
out[407] = 0.27734492538284544;
out[408] = 4.6427621275840973;
out[409] = 0.84818460227515136;
out[410] = -1.3815573406508483;
out[411] = 0.27734492538284594;
out[412] = 1.351252608066726;
out[413] = -0.1189857293249293;
out[414] = 1.3815573406508481;
out[415] = -0.27734492538284544;
out[416] = -10.157468009937038;
out[417] = -22.907008131686908;
out[418] = 6.8962632230037872;
out[419] = 21.781478604028912;
out[435] = -0.1189857293249293;
out[436] = -1.351252608066726;
out[437] = -0.27734492538284544;
out[438] = -1.3815573406508481;
out[439] = -0.84818460227515136;
out[440] = 4.6427621275840973;
out[441] = -0.27734492538284594;
out[442] = -1.3815573406508483;
out[443] = 0.1189857293249293;
out[444] = 1.351252608066726;
out[445] = 0.27734492538284544;
out[446] = 1.3815573406508481;
out[447] = 22.907008131686908;
out[448] = -10.157468009937038;
out[449] = -21.781478604028912;
out[450] = 6.8962632230037872;
out[466] = -1.3815573406508488;
out[467] = 0.27734492538284572;
out[468] = -1.3512526080667266;
out[469] = 0.11898572932492921;
out[470] = -1.3815573406508481;
out[471] = 0.27734492538284605;
out[472] = 4.6427621275840973;
out[473] = 0.84818460227515124;
out[474] = 6.8962632230037881;
out[475] = 21.781478604028912;
out[476] = 6.8659584904196658;
out[477] = 21.939837800086828;
out[478] = 6.8962632230037872;
out[479] = 21.781478604028912;
out[480] = -21.52021310797625;
out[481] = -67.02465519051043;
out[497] = -0.27734492538284572;
out[498] = -1.3815573406508488;
out[499] = -0.11898572932492921;
out[500] = -1.3512526080667266;
out[501] = -0.27734492538284605;
out[502] = -1.3815573406508481;
out[503] = -0.84818460227515124;
out[504] = 4.6427621275840973;
out[505] = -21.781478604028912;
out[506] = 6.8962632230037881;
out[507] = -21.939837800086828;
out[508] = 6.8659584904196658;
out[509] = -21.781478604028912;
out[510] = 6.8962632230037872;
out[511] = 67.02465519051043;
out[512] = -21.52021310797625;
out[536] = -0.063678338513561653;
out[537] = -0.25471335405424661;
out[538] = 0.063678338513561653;
out[539] = 0.25471335405424661;
out[544] = -1;
out[567] = 0.25471335405424661;
out[568] = -0.063678338513561653;
out[569] = -0.25471335405424661;
out[570] = 0.063678338513561653;
out[576] = -1;
out[600] = -0.063678338513561653;
out[601] = -0.25471335405424661;
out[602] = 0.063678338513561653;
out[603] = 0.25471335405424661;
out[608] = -1;
out[631] = 0.25471335405424661;
out[632] = -0.063678338513561653;
out[633] = -0.25471335405424661;
out[634] = 0.063678338513561653;
out[640] = -1;
out[660] = 0.063678338513561653;
out[661] = 0.25471335405424661;
out[664] = -0.063678338513561653;
out[665] = -0.25471335405424661;
out[672] = -1;
out[691] = -0.25471335405424661;
out[692] = 0.063678338513561653;
out[695] = 0.25471335405424661;
out[696] = -0.063678338513561653;
out[704] = -1;
out[922] = 1;
out[924] = 1;
out[926] = 1;
out[928] = 1;
out[954] = 1;
out[956] = 1;
out[958] = 1;
out[960] = 1;

}

void sp_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[293] = y[22];
out[294] = y[23];
out[295] = -y[22];
out[296] = -y[23];
out[297] = y[0] - y[6];
out[298] = y[1] - y[7];
out[299] = y[24];
out[300] = y[25];
out[301] = -y[24];
out[302] = -y[25];
out[303] = y[2] - y[6];
out[304] = y[3] - y[7];
out[305] = y[26];
out[306] = y[27];
out[307] = -y[26];
out[308] = -y[27];
out[309] = y[4] - y[6];
out[310] = y[5] - y[7];
out[311] = -y[23];
out[312] = y[22];
out[313] = y[23];
out[314] = -y[22];
out[315] = y[1] - y[7];
out[316] = -y[0] + y[6];
out[317] = -y[25];
out[318] = y[24];
out[319] = y[25];
out[320] = -y[24];
out[321] = y[3] - y[7];
out[322] = -y[2] + y[6];
out[323] = -y[27];
out[324] = y[26];
out[325] = y[27];
out[326] = -y[26];
out[327] = y[5] - y[7];
out[328] = -y[4] + y[6];

}

void sp_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -1;
out[1] = -4.6427621275840973;
out[2] = -0.84818460227515158;
out[3] = 1.3815573406508483;
out[4] = -0.27734492538284594;
out[5] = 1.3512526080667266;
out[6] = -0.11898572932492885;
out[7] = 1.3815573406508488;
out[8] = -0.27734492538284605;
out[9] = 4.6427621275840973;
out[10] = 0.84818460227515158;
out[11] = -1.3815573406508483;
out[12] = 0.27734492538284594;
out[13] = -1.3512526080667266;
out[14] = 0.11898572932492885;
out[15] = -1.3815573406508488;
out[16] = 0.27734492538284605;
out[17] = 1;
out[18] = 0.84818460227515158;
out[19] = -4.6427621275840973;
out[20] = 0.27734492538284594;
out[21] = 1.3815573406508483;
out[22] = 0.11898572932492885;
out[23] = 1.3512526080667266;
out[24] = 0.27734492538284605;
out[25] = 1.3815573406508488;
out[26] = -0.84818460227515158;
out[27] = 4.6427621275840973;
out[28] = -0.27734492538284594;
out[29] = -1.3815573406508483;
out[30] = -0.11898572932492885;
out[31] = -1.3512526080667266;
out[32] = -0.27734492538284605;
out[33] = -1.3815573406508488;
out[34] = 1;
out[35] = 1.3815573406508483;
out[36] = -0.27734492538284578;
out[37] = -4.6427621275840965;
out[38] = -0.84818460227515136;
out[39] = 1.3815573406508481;
out[40] = -0.27734492538284605;
out[41] = 1.3512526080667262;
out[42] = -0.11898572932492905;
out[43] = -1.3815573406508483;
out[44] = 0.27734492538284578;
out[45] = 4.6427621275840965;
out[46] = 0.84818460227515136;
out[47] = -1.3815573406508481;
out[48] = 0.27734492538284605;
out[49] = -1.3512526080667262;
out[50] = 0.11898572932492905;
out[51] = 1;
out[52] = 0.27734492538284578;
out[53] = 1.3815573406508483;
out[54] = 0.84818460227515136;
out[55] = -4.6427621275840965;
out[56] = 0.27734492538284605;
out[57] = 1.3815573406508481;
out[58] = 0.11898572932492905;
out[59] = 1.3512526080667262;
out[60] = -0.27734492538284578;
out[61] = -1.3815573406508483;
out[62] = -0.84818460227515136;
out[63] = 4.6427621275840965;
out[64] = -0.27734492538284605;
out[65] = -1.3815573406508481;
out[66] = -0.11898572932492905;
out[67] = -1.3512526080667262;
out[68] = 1;
out[69] = 1.351252608066726;
out[70] = -0.1189857293249293;
out[71] = 1.3815573406508481;
out[72] = -0.27734492538284544;
out[73] = -4.6427621275840973;
out[74] = -0.84818460227515136;
out[75] = 1.3815573406508483;
out[76] = -0.27734492538284594;
out[77] = -1.351252608066726;
out[78] = 0.1189857293249293;
out[79] = -1.3815573406508481;
out[80] = 0.27734492538284544;
out[81] = 4.6427621275840973;
out[82] = 0.84818460227515136;
out[83] = -1.3815573406508483;
out[84] = 0.27734492538284594;
out[85] = 1;
out[86] = 0.1189857293249293;
out[87] = 1.351252608066726;
out[88] = 0.27734492538284544;
out[89] = 1.3815573406508481;
out[90] = 0.84818460227515136;
out[91] = -4.6427621275840973;
out[92] = 0.27734492538284594;
out[93] = 1.3815573406508483;
out[94] = -0.1189857293249293;
out[95] = -1.351252608066726;
out[96] = -0.27734492538284544;
out[97] = -1.3815573406508481;
out[98] = -0.84818460227515136;
out[99] = 4.6427621275840973;
out[100] = -0.27734492538284594;
out[101] = -1.3815573406508483;
out[102] = 1;
out[103] = 1.3815573406508488;
out[104] = -0.27734492538284572;
out[105] = 1.3512526080667266;
out[106] = -0.11898572932492921;
out[107] = 1.3815573406508481;
out[108] = -0.27734492538284605;
out[109] = -4.6677621275840977;
out[110] = -0.84818460227515124;
out[111] = -1.3815573406508488;
out[112] = 0.27734492538284572;
out[113] = -1.3512526080667266;
out[114] = 0.11898572932492921;
out[115] = -1.3815573406508481;
out[116] = 0.27734492538284605;
out[117] = 4.6427621275840973;
out[118] = 0.84818460227515124;
out[119] = 0.27734492538284572;
out[120] = 1.3815573406508488;
out[121] = 0.11898572932492921;
out[122] = 1.3512526080667266;
out[123] = 0.27734492538284605;
out[124] = 1.3815573406508481;
out[125] = 0.84818460227515124;
out[126] = -4.6677621275840977;
out[127] = -0.27734492538284572;
out[128] = -1.3815573406508488;
out[129] = -0.11898572932492921;
out[130] = -1.3512526080667266;
out[131] = -0.27734492538284605;
out[132] = -1.3815573406508481;
out[133] = -0.84818460227515124;
out[134] = 4.6427621275840973;
out[135] = 4.6427621275840973;
out[136] = 0.84818460227515158;
out[137] = -1.3815573406508483;
out[138] = 0.27734492538284594;
out[139] = -1.3512526080667266;
out[140] = 0.11898572932492885;
out[141] = -1.3815573406508488;
out[142] = 0.27734492538284605;
out[143] = -10.157468009937038;
out[144] = -22.907008131686908;
out[145] = 1.3815573406508483;
out[146] = -0.27734492538284594;
out[147] = 1.3512526080667266;
out[148] = -0.11898572932492885;
out[149] = 6.8962632230037881;
out[150] = 21.781478604028912;
out[151] = -0.84818460227515158;
out[152] = 4.6427621275840973;
out[153] = -0.27734492538284594;
out[154] = -1.3815573406508483;
out[155] = -0.11898572932492885;
out[156] = -1.3512526080667266;
out[157] = -0.27734492538284605;
out[158] = -1.3815573406508488;
out[159] = 22.907008131686908;
out[160] = -10.157468009937038;
out[161] = 0.27734492538284594;
out[162] = 1.3815573406508483;
out[163] = 0.11898572932492885;
out[164] = 1.3512526080667266;
out[165] = -21.781478604028912;
out[166] = 6.8962632230037881;
out[167] = -1.3815573406508483;
out[168] = 0.27734492538284578;
out[169] = 4.6427621275840965;
out[170] = 0.84818460227515136;
out[171] = -1.3815573406508481;
out[172] = 0.27734492538284605;
out[173] = -1.3512526080667262;
out[174] = 0.11898572932492905;
out[175] = 1.3815573406508483;
out[176] = -0.27734492538284578;
out[177] = -10.157468009937036;
out[178] = -22.907008131686908;
out[179] = 1.3815573406508481;
out[180] = -0.27734492538284605;
out[181] = 6.8659584904196658;
out[182] = 21.939837800086828;
out[183] = -0.27734492538284578;
out[184] = -1.3815573406508483;
out[185] = -0.84818460227515136;
out[186] = 4.6427621275840965;
out[187] = -0.27734492538284605;
out[188] = -1.3815573406508481;
out[189] = -0.11898572932492905;
out[190] = -1.3512526080667262;
out[191] = 0.27734492538284578;
out[192] = 1.3815573406508483;
out[193] = 22.907008131686908;
out[194] = -10.157468009937036;
out[195] = 0.27734492538284605;
out[196] = 1.3815573406508481;
out[197] = -21.939837800086828;
out[198] = 6.8659584904196658;
out[199] = -1.351252608066726;
out[200] = 0.1189857293249293;
out[201] = -1.3815573406508481;
out[202] = 0.27734492538284544;
out[203] = 4.6427621275840973;
out[204] = 0.84818460227515136;
out[205] = -1.3815573406508483;
out[206] = 0.27734492538284594;
out[207] = 1.351252608066726;
out[208] = -0.1189857293249293;
out[209] = 1.3815573406508481;
out[210] = -0.27734492538284544;
out[211] = -10.157468009937038;
out[212] = -22.907008131686908;
out[213] = 6.8962632230037872;
out[214] = 21.781478604028912;
out[215] = -0.1189857293249293;
out[216] = -1.351252608066726;
out[217] = -0.27734492538284544;
out[218] = -1.3815573406508481;
out[219] = -0.84818460227515136;
out[220] = 4.6427621275840973;
out[221] = -0.27734492538284594;
out[222] = -1.3815573406508483;
out[223] = 0.1189857293249293;
out[224] = 1.351252608066726;
out[225] = 0.27734492538284544;
out[226] = 1.3815573406508481;
out[227] = 22.907008131686908;
out[228] = -10.157468009937038;
out[229] = -21.781478604028912;
out[230] = 6.8962632230037872;
out[231] = -1.3815573406508488;
out[232] = 0.27734492538284572;
out[233] = -1.3512526080667266;
out[234] = 0.11898572932492921;
out[235] = -1.3815573406508481;
out[236] = 0.27734492538284605;
out[237] = 4.6427621275840973;
out[238] = 0.84818460227515124;
out[239] = 6.8962632230037881;
out[240] = 21.781478604028912;
out[241] = 6.8659584904196658;
out[242] = 21.939837800086828;
out[243] = 6.8962632230037872;
out[244] = 21.781478604028912;
out[245] = -21.52021310797625;
out[246] = -67.02465519051043;
out[247] = -0.27734492538284572;
out[248] = -1.3815573406508488;
out[249] = -0.11898572932492921;
out[250] = -1.3512526080667266;
out[251] = -0.27734492538284605;
out[252] = -1.3815573406508481;
out[253] = -0.84818460227515124;
out[254] = 4.6427621275840973;
out[255] = -21.781478604028912;
out[256] = 6.8962632230037881;
out[257] = -21.939837800086828;
out[258] = 6.8659584904196658;
out[259] = -21.781478604028912;
out[260] = 6.8962632230037872;
out[261] = 67.02465519051043;
out[262] = -21.52021310797625;
out[263] = -0.063678338513561653;
out[264] = -0.25471335405424661;
out[265] = 0.063678338513561653;
out[266] = 0.25471335405424661;
out[267] = -1;
out[268] = 0.25471335405424661;
out[269] = -0.063678338513561653;
out[270] = -0.25471335405424661;
out[271] = 0.063678338513561653;
out[272] = -1;
out[273] = -0.063678338513561653;
out[274] = -0.25471335405424661;
out[275] = 0.063678338513561653;
out[276] = 0.25471335405424661;
out[277] = -1;
out[278] = 0.25471335405424661;
out[279] = -0.063678338513561653;
out[280] = -0.25471335405424661;
out[281] = 0.063678338513561653;
out[282] = -1;
out[283] = 0.063678338513561653;
out[284] = 0.25471335405424661;
out[285] = -0.063678338513561653;
out[286] = -0.25471335405424661;
out[287] = -1;
out[288] = -0.25471335405424661;
out[289] = 0.063678338513561653;
out[290] = 0.25471335405424661;
out[291] = -0.063678338513561653;
out[292] = -1;
out[329] = 1;
out[330] = 1;
out[331] = 1;
out[332] = 1;
out[333] = 1;
out[334] = 1;
out[335] = 1;
out[336] = 1;

}

void de_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[714] = y[22];
out[715] = y[23];
out[720] = -y[22];
out[721] = -y[23];
out[736] = y[0] - y[6];
out[737] = y[1] - y[7];
out[747] = y[24];
out[748] = y[25];
out[751] = -y[24];
out[752] = -y[25];
out[769] = y[2] - y[6];
out[770] = y[3] - y[7];
out[780] = y[26];
out[781] = y[27];
out[782] = -y[26];
out[783] = -y[27];
out[802] = y[4] - y[6];
out[803] = y[5] - y[7];
out[807] = -y[23];
out[808] = y[22];
out[813] = y[23];
out[814] = -y[22];
out[829] = y[1] - y[7];
out[830] = -y[0] + y[6];
out[840] = -y[25];
out[841] = y[24];
out[844] = y[25];
out[845] = -y[24];
out[862] = y[3] - y[7];
out[863] = -y[2] + y[6];
out[873] = -y[27];
out[874] = y[26];
out[875] = y[27];
out[876] = -y[26];
out[895] = y[5] - y[7];
out[896] = -y[4] + y[6];

}

void de_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt + 1;

}

void de_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[32] = -4.6427621275840973;
out[33] = -0.84818460227515158;
out[34] = 1.3815573406508483;
out[35] = -0.27734492538284594;
out[36] = 1.3512526080667266;
out[37] = -0.11898572932492885;
out[38] = 1.3815573406508488;
out[39] = -0.27734492538284605;
out[40] = 4.6427621275840973;
out[41] = 0.84818460227515158;
out[42] = -1.3815573406508483;
out[43] = 0.27734492538284594;
out[44] = -1.3512526080667266;
out[45] = 0.11898572932492885;
out[46] = -1.3815573406508488;
out[47] = 0.27734492538284605;
out[54] = 1;
out[63] = 0.84818460227515158;
out[64] = -4.6427621275840973;
out[65] = 0.27734492538284594;
out[66] = 1.3815573406508483;
out[67] = 0.11898572932492885;
out[68] = 1.3512526080667266;
out[69] = 0.27734492538284605;
out[70] = 1.3815573406508488;
out[71] = -0.84818460227515158;
out[72] = 4.6427621275840973;
out[73] = -0.27734492538284594;
out[74] = -1.3815573406508483;
out[75] = -0.11898572932492885;
out[76] = -1.3512526080667266;
out[77] = -0.27734492538284605;
out[78] = -1.3815573406508488;
out[86] = 1;
out[94] = 1.3815573406508483;
out[95] = -0.27734492538284578;
out[96] = -4.6427621275840965;
out[97] = -0.84818460227515136;
out[98] = 1.3815573406508481;
out[99] = -0.27734492538284605;
out[100] = 1.3512526080667262;
out[101] = -0.11898572932492905;
out[102] = -1.3815573406508483;
out[103] = 0.27734492538284578;
out[104] = 4.6427621275840965;
out[105] = 0.84818460227515136;
out[106] = -1.3815573406508481;
out[107] = 0.27734492538284605;
out[108] = -1.3512526080667262;
out[109] = 0.11898572932492905;
out[118] = 1;
out[125] = 0.27734492538284578;
out[126] = 1.3815573406508483;
out[127] = 0.84818460227515136;
out[128] = -4.6427621275840965;
out[129] = 0.27734492538284605;
out[130] = 1.3815573406508481;
out[131] = 0.11898572932492905;
out[132] = 1.3512526080667262;
out[133] = -0.27734492538284578;
out[134] = -1.3815573406508483;
out[135] = -0.84818460227515136;
out[136] = 4.6427621275840965;
out[137] = -0.27734492538284605;
out[138] = -1.3815573406508481;
out[139] = -0.11898572932492905;
out[140] = -1.3512526080667262;
out[150] = 1;
out[156] = 1.351252608066726;
out[157] = -0.1189857293249293;
out[158] = 1.3815573406508481;
out[159] = -0.27734492538284544;
out[160] = -4.6427621275840973;
out[161] = -0.84818460227515136;
out[162] = 1.3815573406508483;
out[163] = -0.27734492538284594;
out[164] = -1.351252608066726;
out[165] = 0.1189857293249293;
out[166] = -1.3815573406508481;
out[167] = 0.27734492538284544;
out[168] = 4.6427621275840973;
out[169] = 0.84818460227515136;
out[170] = -1.3815573406508483;
out[171] = 0.27734492538284594;
out[182] = 1;
out[187] = 0.1189857293249293;
out[188] = 1.351252608066726;
out[189] = 0.27734492538284544;
out[190] = 1.3815573406508481;
out[191] = 0.84818460227515136;
out[192] = -4.6427621275840973;
out[193] = 0.27734492538284594;
out[194] = 1.3815573406508483;
out[195] = -0.1189857293249293;
out[196] = -1.351252608066726;
out[197] = -0.27734492538284544;
out[198] = -1.3815573406508481;
out[199] = -0.84818460227515136;
out[200] = 4.6427621275840973;
out[201] = -0.27734492538284594;
out[202] = -1.3815573406508483;
out[214] = 1;
out[218] = 1.3815573406508488;
out[219] = -0.27734492538284572;
out[220] = 1.3512526080667266;
out[221] = -0.11898572932492921;
out[222] = 1.3815573406508481;
out[223] = -0.27734492538284605;
out[224] = -4.6677621275840977;
out[225] = -0.84818460227515124;
out[226] = -1.3815573406508488;
out[227] = 0.27734492538284572;
out[228] = -1.3512526080667266;
out[229] = 0.11898572932492921;
out[230] = -1.3815573406508481;
out[231] = 0.27734492538284605;
out[232] = 4.6427621275840973;
out[233] = 0.84818460227515124;
out[249] = 0.27734492538284572;
out[250] = 1.3815573406508488;
out[251] = 0.11898572932492921;
out[252] = 1.3512526080667266;
out[253] = 0.27734492538284605;
out[254] = 1.3815573406508481;
out[255] = 0.84818460227515124;
out[256] = -4.6677621275840977;
out[257] = -0.27734492538284572;
out[258] = -1.3815573406508488;
out[259] = -0.11898572932492921;
out[260] = -1.3512526080667266;
out[261] = -0.27734492538284605;
out[262] = -1.3815573406508481;
out[263] = -0.84818460227515124;
out[264] = 4.6427621275840973;
out[280] = 4.6427621275840973;
out[281] = 0.84818460227515158;
out[282] = -1.3815573406508483;
out[283] = 0.27734492538284594;
out[284] = -1.3512526080667266;
out[285] = 0.11898572932492885;
out[286] = -1.3815573406508488;
out[287] = 0.27734492538284605;
out[288] = -10.157468009937038;
out[289] = -22.907008131686908;
out[290] = 1.3815573406508483;
out[291] = -0.27734492538284594;
out[292] = 1.3512526080667266;
out[293] = -0.11898572932492885;
out[294] = 6.8962632230037881;
out[295] = 21.781478604028912;
out[311] = -0.84818460227515158;
out[312] = 4.6427621275840973;
out[313] = -0.27734492538284594;
out[314] = -1.3815573406508483;
out[315] = -0.11898572932492885;
out[316] = -1.3512526080667266;
out[317] = -0.27734492538284605;
out[318] = -1.3815573406508488;
out[319] = 22.907008131686908;
out[320] = -10.157468009937038;
out[321] = 0.27734492538284594;
out[322] = 1.3815573406508483;
out[323] = 0.11898572932492885;
out[324] = 1.3512526080667266;
out[325] = -21.781478604028912;
out[326] = 6.8962632230037881;
out[342] = -1.3815573406508483;
out[343] = 0.27734492538284578;
out[344] = 4.6427621275840965;
out[345] = 0.84818460227515136;
out[346] = -1.3815573406508481;
out[347] = 0.27734492538284605;
out[348] = -1.3512526080667262;
out[349] = 0.11898572932492905;
out[350] = 1.3815573406508483;
out[351] = -0.27734492538284578;
out[352] = -10.157468009937036;
out[353] = -22.907008131686908;
out[354] = 1.3815573406508481;
out[355] = -0.27734492538284605;
out[356] = 6.8659584904196658;
out[357] = 21.939837800086828;
out[373] = -0.27734492538284578;
out[374] = -1.3815573406508483;
out[375] = -0.84818460227515136;
out[376] = 4.6427621275840965;
out[377] = -0.27734492538284605;
out[378] = -1.3815573406508481;
out[379] = -0.11898572932492905;
out[380] = -1.3512526080667262;
out[381] = 0.27734492538284578;
out[382] = 1.3815573406508483;
out[383] = 22.907008131686908;
out[384] = -10.157468009937036;
out[385] = 0.27734492538284605;
out[386] = 1.3815573406508481;
out[387] = -21.939837800086828;
out[388] = 6.8659584904196658;
out[404] = -1.351252608066726;
out[405] = 0.1189857293249293;
out[406] = -1.3815573406508481;
out[407] = 0.27734492538284544;
out[408] = 4.6427621275840973;
out[409] = 0.84818460227515136;
out[410] = -1.3815573406508483;
out[411] = 0.27734492538284594;
out[412] = 1.351252608066726;
out[413] = -0.1189857293249293;
out[414] = 1.3815573406508481;
out[415] = -0.27734492538284544;
out[416] = -10.157468009937038;
out[417] = -22.907008131686908;
out[418] = 6.8962632230037872;
out[419] = 21.781478604028912;
out[435] = -0.1189857293249293;
out[436] = -1.351252608066726;
out[437] = -0.27734492538284544;
out[438] = -1.3815573406508481;
out[439] = -0.84818460227515136;
out[440] = 4.6427621275840973;
out[441] = -0.27734492538284594;
out[442] = -1.3815573406508483;
out[443] = 0.1189857293249293;
out[444] = 1.351252608066726;
out[445] = 0.27734492538284544;
out[446] = 1.3815573406508481;
out[447] = 22.907008131686908;
out[448] = -10.157468009937038;
out[449] = -21.781478604028912;
out[450] = 6.8962632230037872;
out[466] = -1.3815573406508488;
out[467] = 0.27734492538284572;
out[468] = -1.3512526080667266;
out[469] = 0.11898572932492921;
out[470] = -1.3815573406508481;
out[471] = 0.27734492538284605;
out[472] = 4.6427621275840973;
out[473] = 0.84818460227515124;
out[474] = 6.8962632230037881;
out[475] = 21.781478604028912;
out[476] = 6.8659584904196658;
out[477] = 21.939837800086828;
out[478] = 6.8962632230037872;
out[479] = 21.781478604028912;
out[480] = -21.52021310797625;
out[481] = -67.02465519051043;
out[497] = -0.27734492538284572;
out[498] = -1.3815573406508488;
out[499] = -0.11898572932492921;
out[500] = -1.3512526080667266;
out[501] = -0.27734492538284605;
out[502] = -1.3815573406508481;
out[503] = -0.84818460227515124;
out[504] = 4.6427621275840973;
out[505] = -21.781478604028912;
out[506] = 6.8962632230037881;
out[507] = -21.939837800086828;
out[508] = 6.8659584904196658;
out[509] = -21.781478604028912;
out[510] = 6.8962632230037872;
out[511] = 67.02465519051043;
out[512] = -21.52021310797625;
out[536] = -0.063678338513561653;
out[537] = -0.25471335405424661;
out[538] = 0.063678338513561653;
out[539] = 0.25471335405424661;
out[544] = -1;
out[567] = 0.25471335405424661;
out[568] = -0.063678338513561653;
out[569] = -0.25471335405424661;
out[570] = 0.063678338513561653;
out[576] = -1;
out[600] = -0.063678338513561653;
out[601] = -0.25471335405424661;
out[602] = 0.063678338513561653;
out[603] = 0.25471335405424661;
out[608] = -1;
out[631] = 0.25471335405424661;
out[632] = -0.063678338513561653;
out[633] = -0.25471335405424661;
out[634] = 0.063678338513561653;
out[640] = -1;
out[660] = 0.063678338513561653;
out[661] = 0.25471335405424661;
out[664] = -0.063678338513561653;
out[665] = -0.25471335405424661;
out[672] = -1;
out[691] = -0.25471335405424661;
out[692] = 0.063678338513561653;
out[695] = 0.25471335405424661;
out[696] = -0.063678338513561653;
out[704] = -1;
out[922] = 1;
out[924] = 1;
out[926] = 1;
out[928] = 1;
out[954] = 1;
out[956] = 1;
out[958] = 1;
out[960] = 1;

}

void sp_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[293] = y[22];
out[294] = y[23];
out[295] = -y[22];
out[296] = -y[23];
out[297] = y[0] - y[6];
out[298] = y[1] - y[7];
out[299] = y[24];
out[300] = y[25];
out[301] = -y[24];
out[302] = -y[25];
out[303] = y[2] - y[6];
out[304] = y[3] - y[7];
out[305] = y[26];
out[306] = y[27];
out[307] = -y[26];
out[308] = -y[27];
out[309] = y[4] - y[6];
out[310] = y[5] - y[7];
out[311] = -y[23];
out[312] = y[22];
out[313] = y[23];
out[314] = -y[22];
out[315] = y[1] - y[7];
out[316] = -y[0] + y[6];
out[317] = -y[25];
out[318] = y[24];
out[319] = y[25];
out[320] = -y[24];
out[321] = y[3] - y[7];
out[322] = -y[2] + y[6];
out[323] = -y[27];
out[324] = y[26];
out[325] = y[27];
out[326] = -y[26];
out[327] = y[5] - y[7];
out[328] = -y[4] + y[6];

}

void sp_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt + 1;

}

void sp_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1] = -4.6427621275840973;
out[2] = -0.84818460227515158;
out[3] = 1.3815573406508483;
out[4] = -0.27734492538284594;
out[5] = 1.3512526080667266;
out[6] = -0.11898572932492885;
out[7] = 1.3815573406508488;
out[8] = -0.27734492538284605;
out[9] = 4.6427621275840973;
out[10] = 0.84818460227515158;
out[11] = -1.3815573406508483;
out[12] = 0.27734492538284594;
out[13] = -1.3512526080667266;
out[14] = 0.11898572932492885;
out[15] = -1.3815573406508488;
out[16] = 0.27734492538284605;
out[17] = 1;
out[18] = 0.84818460227515158;
out[19] = -4.6427621275840973;
out[20] = 0.27734492538284594;
out[21] = 1.3815573406508483;
out[22] = 0.11898572932492885;
out[23] = 1.3512526080667266;
out[24] = 0.27734492538284605;
out[25] = 1.3815573406508488;
out[26] = -0.84818460227515158;
out[27] = 4.6427621275840973;
out[28] = -0.27734492538284594;
out[29] = -1.3815573406508483;
out[30] = -0.11898572932492885;
out[31] = -1.3512526080667266;
out[32] = -0.27734492538284605;
out[33] = -1.3815573406508488;
out[34] = 1;
out[35] = 1.3815573406508483;
out[36] = -0.27734492538284578;
out[37] = -4.6427621275840965;
out[38] = -0.84818460227515136;
out[39] = 1.3815573406508481;
out[40] = -0.27734492538284605;
out[41] = 1.3512526080667262;
out[42] = -0.11898572932492905;
out[43] = -1.3815573406508483;
out[44] = 0.27734492538284578;
out[45] = 4.6427621275840965;
out[46] = 0.84818460227515136;
out[47] = -1.3815573406508481;
out[48] = 0.27734492538284605;
out[49] = -1.3512526080667262;
out[50] = 0.11898572932492905;
out[51] = 1;
out[52] = 0.27734492538284578;
out[53] = 1.3815573406508483;
out[54] = 0.84818460227515136;
out[55] = -4.6427621275840965;
out[56] = 0.27734492538284605;
out[57] = 1.3815573406508481;
out[58] = 0.11898572932492905;
out[59] = 1.3512526080667262;
out[60] = -0.27734492538284578;
out[61] = -1.3815573406508483;
out[62] = -0.84818460227515136;
out[63] = 4.6427621275840965;
out[64] = -0.27734492538284605;
out[65] = -1.3815573406508481;
out[66] = -0.11898572932492905;
out[67] = -1.3512526080667262;
out[68] = 1;
out[69] = 1.351252608066726;
out[70] = -0.1189857293249293;
out[71] = 1.3815573406508481;
out[72] = -0.27734492538284544;
out[73] = -4.6427621275840973;
out[74] = -0.84818460227515136;
out[75] = 1.3815573406508483;
out[76] = -0.27734492538284594;
out[77] = -1.351252608066726;
out[78] = 0.1189857293249293;
out[79] = -1.3815573406508481;
out[80] = 0.27734492538284544;
out[81] = 4.6427621275840973;
out[82] = 0.84818460227515136;
out[83] = -1.3815573406508483;
out[84] = 0.27734492538284594;
out[85] = 1;
out[86] = 0.1189857293249293;
out[87] = 1.351252608066726;
out[88] = 0.27734492538284544;
out[89] = 1.3815573406508481;
out[90] = 0.84818460227515136;
out[91] = -4.6427621275840973;
out[92] = 0.27734492538284594;
out[93] = 1.3815573406508483;
out[94] = -0.1189857293249293;
out[95] = -1.351252608066726;
out[96] = -0.27734492538284544;
out[97] = -1.3815573406508481;
out[98] = -0.84818460227515136;
out[99] = 4.6427621275840973;
out[100] = -0.27734492538284594;
out[101] = -1.3815573406508483;
out[102] = 1;
out[103] = 1.3815573406508488;
out[104] = -0.27734492538284572;
out[105] = 1.3512526080667266;
out[106] = -0.11898572932492921;
out[107] = 1.3815573406508481;
out[108] = -0.27734492538284605;
out[109] = -4.6677621275840977;
out[110] = -0.84818460227515124;
out[111] = -1.3815573406508488;
out[112] = 0.27734492538284572;
out[113] = -1.3512526080667266;
out[114] = 0.11898572932492921;
out[115] = -1.3815573406508481;
out[116] = 0.27734492538284605;
out[117] = 4.6427621275840973;
out[118] = 0.84818460227515124;
out[119] = 0.27734492538284572;
out[120] = 1.3815573406508488;
out[121] = 0.11898572932492921;
out[122] = 1.3512526080667266;
out[123] = 0.27734492538284605;
out[124] = 1.3815573406508481;
out[125] = 0.84818460227515124;
out[126] = -4.6677621275840977;
out[127] = -0.27734492538284572;
out[128] = -1.3815573406508488;
out[129] = -0.11898572932492921;
out[130] = -1.3512526080667266;
out[131] = -0.27734492538284605;
out[132] = -1.3815573406508481;
out[133] = -0.84818460227515124;
out[134] = 4.6427621275840973;
out[135] = 4.6427621275840973;
out[136] = 0.84818460227515158;
out[137] = -1.3815573406508483;
out[138] = 0.27734492538284594;
out[139] = -1.3512526080667266;
out[140] = 0.11898572932492885;
out[141] = -1.3815573406508488;
out[142] = 0.27734492538284605;
out[143] = -10.157468009937038;
out[144] = -22.907008131686908;
out[145] = 1.3815573406508483;
out[146] = -0.27734492538284594;
out[147] = 1.3512526080667266;
out[148] = -0.11898572932492885;
out[149] = 6.8962632230037881;
out[150] = 21.781478604028912;
out[151] = -0.84818460227515158;
out[152] = 4.6427621275840973;
out[153] = -0.27734492538284594;
out[154] = -1.3815573406508483;
out[155] = -0.11898572932492885;
out[156] = -1.3512526080667266;
out[157] = -0.27734492538284605;
out[158] = -1.3815573406508488;
out[159] = 22.907008131686908;
out[160] = -10.157468009937038;
out[161] = 0.27734492538284594;
out[162] = 1.3815573406508483;
out[163] = 0.11898572932492885;
out[164] = 1.3512526080667266;
out[165] = -21.781478604028912;
out[166] = 6.8962632230037881;
out[167] = -1.3815573406508483;
out[168] = 0.27734492538284578;
out[169] = 4.6427621275840965;
out[170] = 0.84818460227515136;
out[171] = -1.3815573406508481;
out[172] = 0.27734492538284605;
out[173] = -1.3512526080667262;
out[174] = 0.11898572932492905;
out[175] = 1.3815573406508483;
out[176] = -0.27734492538284578;
out[177] = -10.157468009937036;
out[178] = -22.907008131686908;
out[179] = 1.3815573406508481;
out[180] = -0.27734492538284605;
out[181] = 6.8659584904196658;
out[182] = 21.939837800086828;
out[183] = -0.27734492538284578;
out[184] = -1.3815573406508483;
out[185] = -0.84818460227515136;
out[186] = 4.6427621275840965;
out[187] = -0.27734492538284605;
out[188] = -1.3815573406508481;
out[189] = -0.11898572932492905;
out[190] = -1.3512526080667262;
out[191] = 0.27734492538284578;
out[192] = 1.3815573406508483;
out[193] = 22.907008131686908;
out[194] = -10.157468009937036;
out[195] = 0.27734492538284605;
out[196] = 1.3815573406508481;
out[197] = -21.939837800086828;
out[198] = 6.8659584904196658;
out[199] = -1.351252608066726;
out[200] = 0.1189857293249293;
out[201] = -1.3815573406508481;
out[202] = 0.27734492538284544;
out[203] = 4.6427621275840973;
out[204] = 0.84818460227515136;
out[205] = -1.3815573406508483;
out[206] = 0.27734492538284594;
out[207] = 1.351252608066726;
out[208] = -0.1189857293249293;
out[209] = 1.3815573406508481;
out[210] = -0.27734492538284544;
out[211] = -10.157468009937038;
out[212] = -22.907008131686908;
out[213] = 6.8962632230037872;
out[214] = 21.781478604028912;
out[215] = -0.1189857293249293;
out[216] = -1.351252608066726;
out[217] = -0.27734492538284544;
out[218] = -1.3815573406508481;
out[219] = -0.84818460227515136;
out[220] = 4.6427621275840973;
out[221] = -0.27734492538284594;
out[222] = -1.3815573406508483;
out[223] = 0.1189857293249293;
out[224] = 1.351252608066726;
out[225] = 0.27734492538284544;
out[226] = 1.3815573406508481;
out[227] = 22.907008131686908;
out[228] = -10.157468009937038;
out[229] = -21.781478604028912;
out[230] = 6.8962632230037872;
out[231] = -1.3815573406508488;
out[232] = 0.27734492538284572;
out[233] = -1.3512526080667266;
out[234] = 0.11898572932492921;
out[235] = -1.3815573406508481;
out[236] = 0.27734492538284605;
out[237] = 4.6427621275840973;
out[238] = 0.84818460227515124;
out[239] = 6.8962632230037881;
out[240] = 21.781478604028912;
out[241] = 6.8659584904196658;
out[242] = 21.939837800086828;
out[243] = 6.8962632230037872;
out[244] = 21.781478604028912;
out[245] = -21.52021310797625;
out[246] = -67.02465519051043;
out[247] = -0.27734492538284572;
out[248] = -1.3815573406508488;
out[249] = -0.11898572932492921;
out[250] = -1.3512526080667266;
out[251] = -0.27734492538284605;
out[252] = -1.3815573406508481;
out[253] = -0.84818460227515124;
out[254] = 4.6427621275840973;
out[255] = -21.781478604028912;
out[256] = 6.8962632230037881;
out[257] = -21.939837800086828;
out[258] = 6.8659584904196658;
out[259] = -21.781478604028912;
out[260] = 6.8962632230037872;
out[261] = 67.02465519051043;
out[262] = -21.52021310797625;
out[263] = -0.063678338513561653;
out[264] = -0.25471335405424661;
out[265] = 0.063678338513561653;
out[266] = 0.25471335405424661;
out[267] = -1;
out[268] = 0.25471335405424661;
out[269] = -0.063678338513561653;
out[270] = -0.25471335405424661;
out[271] = 0.063678338513561653;
out[272] = -1;
out[273] = -0.063678338513561653;
out[274] = -0.25471335405424661;
out[275] = 0.063678338513561653;
out[276] = 0.25471335405424661;
out[277] = -1;
out[278] = 0.25471335405424661;
out[279] = -0.063678338513561653;
out[280] = -0.25471335405424661;
out[281] = 0.063678338513561653;
out[282] = -1;
out[283] = 0.063678338513561653;
out[284] = 0.25471335405424661;
out[285] = -0.063678338513561653;
out[286] = -0.25471335405424661;
out[287] = -1;
out[288] = -0.25471335405424661;
out[289] = 0.063678338513561653;
out[290] = 0.25471335405424661;
out[291] = -0.063678338513561653;
out[292] = -1;
out[329] = 1;
out[330] = 1;
out[331] = 1;
out[332] = 1;
out[333] = 1;
out[334] = 1;
out[335] = 1;
out[336] = 1;

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_num_eval _cffi_d_de_jac_ini_num_eval
#endif

static void _cffi_d_de_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_up_eval _cffi_d_de_jac_ini_up_eval
#endif

static void _cffi_d_de_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_xy_eval _cffi_d_de_jac_ini_xy_eval
#endif

static void _cffi_d_de_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_num_eval _cffi_d_de_jac_trap_num_eval
#endif

static void _cffi_d_de_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_up_eval _cffi_d_de_jac_trap_up_eval
#endif

static void _cffi_d_de_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_xy_eval _cffi_d_de_jac_trap_xy_eval
#endif

static void _cffi_d_f_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_ini_eval _cffi_d_f_ini_eval
#endif

static void _cffi_d_f_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_run_eval _cffi_d_f_run_eval
#endif

static void _cffi_d_g_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_ini_eval _cffi_d_g_ini_eval
#endif

static void _cffi_d_g_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_run_eval _cffi_d_g_run_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static void _cffi_d_sp_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_num_eval _cffi_d_sp_jac_ini_num_eval
#endif

static void _cffi_d_sp_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_up_eval _cffi_d_sp_jac_ini_up_eval
#endif

static void _cffi_d_sp_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_xy_eval _cffi_d_sp_jac_ini_xy_eval
#endif

static void _cffi_d_sp_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_num_eval _cffi_d_sp_jac_trap_num_eval
#endif

static void _cffi_d_sp_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_up_eval _cffi_d_sp_jac_trap_up_eval
#endif

static void _cffi_d_sp_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_xy_eval _cffi_d_sp_jac_trap_xy_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_jac_ini_num_eval", (void *)_cffi_f_de_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_num_eval },
  { "de_jac_ini_up_eval", (void *)_cffi_f_de_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_up_eval },
  { "de_jac_ini_xy_eval", (void *)_cffi_f_de_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_xy_eval },
  { "de_jac_trap_num_eval", (void *)_cffi_f_de_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_num_eval },
  { "de_jac_trap_up_eval", (void *)_cffi_f_de_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_up_eval },
  { "de_jac_trap_xy_eval", (void *)_cffi_f_de_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_xy_eval },
  { "f_ini_eval", (void *)_cffi_f_f_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_ini_eval },
  { "f_run_eval", (void *)_cffi_f_f_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_run_eval },
  { "g_ini_eval", (void *)_cffi_f_g_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_ini_eval },
  { "g_run_eval", (void *)_cffi_f_g_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_run_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
  { "sp_jac_ini_num_eval", (void *)_cffi_f_sp_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_num_eval },
  { "sp_jac_ini_up_eval", (void *)_cffi_f_sp_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_up_eval },
  { "sp_jac_ini_xy_eval", (void *)_cffi_f_sp_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_xy_eval },
  { "sp_jac_trap_num_eval", (void *)_cffi_f_sp_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_num_eval },
  { "sp_jac_trap_up_eval", (void *)_cffi_f_sp_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_up_eval },
  { "sp_jac_trap_xy_eval", (void *)_cffi_f_sp_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_xy_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  17,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_cigre_eu_lv_ind_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_cigre_eu_lv_ind_cffi(void) { return NULL; }
#  else
     initcigre_eu_lv_ind_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_cigre_eu_lv_ind_cffi(void)
{
  return _cffi_init("cigre_eu_lv_ind_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
initcigre_eu_lv_ind_cffi(void)
{
  _cffi_init("cigre_eu_lv_ind_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
